package it.andynaz.log;

import it.andynaz.utils.Utils;

/**
 * Logger di base.
 * 
 * Il Logger si occupa di registrare materialmente i log. Questo Logger utilizza
 * il canale <tt>System.out</tt>.
 * 
 * <p>Ogni riga del log viene scritta nel formato
 * <tt>[data::][nome::]log</tt> (data e nome potrebbero non essere
 * presenti).</p>
 * 
 * <p>Il Logger ha una propria configurazione personale {@link #conf}; in caso
 * non sia presente viene usata la configrazione restituita dal metodo {@link
 * Config#getConfig()}. Questo permette di cambiare la configurazione di tutti i
 * Logger privi di configurazione personale facilmente.</p>
 * 
 * <p>I log vengono scritti dai vari metodi <tt>log</tt>: ogni metodo controlla
 * prima della scrittura che il livello della configurazione sia sufficiente a
 * scrivere il log.<br/>
 * Sottoclassi di questa classe possono cambiare la modalit&agrave; di scrittura
 * del log (ad esempio su un file o su un DB) ri-definendo il solo metodo {@link
 * #log(String, Level)}. In caso di necessit&agrave; (ad esempio per maggiore
 * efficienza) potrebbero anche ri-definirne altri.</p>
 * 
 * @author andynaz
 * @version 0.1 2012/08/20
 */
public class Logger{
	
	/**
	 * Separatore delle varie sezioni del log.
	 * 
	 * Viene usato per separare data, nome e log.
	 * 
	 * @since 0.1
	 */
	public static String separator = "::";
	
	/**
	 * Nome del Logger.
	 * 
	 * Se presente viene anteposto al log.
	 * 
	 * @since 0.1
	 */
	private String name;
	
	/**
	 * Configurazione usata dal Logger.
	 * 
	 * Nel caso sia <tt>null</tt> usa quella restituita dal metodo {@link
	 * Config#getConfig()}.
	 * 
	 * @since 0.1
	 */
	private Config conf;
	
	
	/**
	 * Costruisce un Logger con un nome e una configurazione.
	 * 
	 * @param name nome del Logger
	 * @param conf configurazione del Logger
	 * 
	 * @since 0.1
	 */
	public Logger(String name, Config conf){
		this.name = name;
		this.conf = conf;
	}
	
	/**
	 * Costruisce un Logger con un nome.
	 * 
	 * @param name nome del Logger
	 * 
	 * @since 0.1
	 */
	public Logger(String name){
		this(name, null);
	}
	
	/**
	 * Costruisce un Logger con una configurazione.
	 * 
	 * @param conf configurazione del Logger
	 * 
	 * @since 0.1
	 */
	public Logger(Config conf){
		this(null, conf);
	}
	
	/**
	 * Costruisce un Logger.
	 * 
	 * @since 0.1
	 */
	public Logger(){
		this(null, null);
	}
	
	
	/**
	 * Scrive una riga nel log con un livello personalizzato.
	 * 
	 * @param log log da scrivere
	 * @param level livello del log
	 * 
	 * @since 0.1
	 */
	public void log(String log, Level level){
		Config inUseConf = getInUseConf();
		if( inUseConf.getLevel().hasToLog(level) ){
			System.out.println(getLogPrefix() + log);
		}
	}

	/**
	 * Scrive una riga nel log.
	 * 
	 * Il livello utilizzato &egrave; {@link Level#INFO INFO}.
	 * 
	 * @param log log da scrivere
	 * 
	 * @since 0.1
	 */
	public void log(String log){
		log(log, Level.INFO);
	}
	
	/**
	 * Scrive un'eccezione nel log.
	 * 
	 * Il livello utilizzato &egrave; {@link Level#INFO INFO}.
	 * 
	 * @param e eccezione da loggare
	 * 
	 * @since 0.1
	 */
	public void log(Exception e){
		log(e, Level.INFO);
	}
	
	/**
	 * Scrive un'eccezione nel log con un livello personalizzato.
	 * 
	 * <p>Viene scritto il tipo di eccezione (con il messaggio dell'eccezione, se
	 * presente) e lo stack trace.</p>
	 * 
	 * @param e eccezione da loggare
	 * @param level livello del log
	 * 
	 * @since 0.1
	 */
	public void log(Exception e, Level level){
		log(e.toString(), level);
		for( StackTraceElement ste : e.getStackTrace() )
			log("    "+ste.toString(), level);
		Throwable t = e.getCause();
		if( t!=null && t instanceof Exception ){
			log("  caused by:", level);
			log((Exception)t, level);
		}
	}
	
	/**
	 * Restituisce la configurazione in uso al Logger.
	 * 
	 * Nel caso in cui il Logger non abbia una configurazione, viene usata quella
	 * restituita dal metodo {@link Config#getConfig()}.
	 * 
	 * @return la configurazione attualmente utilizzata da questo Logger.
	 * 
	 * @since 0.1
	 */
	protected Config getInUseConf(){
		return this.conf!=null ? this.conf : Config.getConfig() ;
	}
	
	/**
	 * Restituisce il prefisso dal anteporre al log.
	 * 
	 * Il prefisso viene costruito in base al nome del Logger corrente e alle
	 * impostazioni correnti.
	 * 
	 * <p>Il prefisso &egrave; del tipo <tt>[data::][nome::]</tt>.</p>
	 * 
	 * @return il prefisso dal anteporre al log
	 * 
	 * @since 0.1
	 */
	protected String getLogPrefix(){
		Config inUseConf = getInUseConf();
		String logPrefix = "";
		if( !Utils.isEmpty(this.name) )
			logPrefix = this.name + Logger.separator + logPrefix;
		if( inUseConf.isWithDate() )
			logPrefix = inUseConf.getDate() + Logger.separator + logPrefix;
		return logPrefix;
	}
	
}
